/********************************************************************************/
/*                                                                              */
/*     Happystats $Id: cgi_xmlparser.c,v 1.2 2004/01/25 00:29:22 ofredj Exp $	*/
/*                                                                              */
/*     This program is free software; you can redistribute it and/or modify     */
/*     it under the terms of the GNU General Public License as published by     */
/*     the Free Software Foundation; either version 2 of the License, or        */
/*     (at your option) any later version.                                      */
/*                                                                              */
/*     This program is distributed in the hope that it will be useful,          */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            */
/*     GNU General Public License for more details.                             */
/*                                                                              */
/*     You should have received a copy of the GNU General Public License        */
/*     along with this program; if not, write to the Free Software              */
/*     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
/*                                                                              */
/********************************************************************************/

/* Includes */
#include "happystats.h"
#include "cgi.h"

void printgraphds (graphDsPtr ds) ;
char *freegraphds(graphDsPtr graphDs) ;
char *freegraph(graphPtr graph) ; 
graphPtr parseGraph(xmlDocPtr doc, xmlNodePtr cur) ;
graphDsPtr parseGraphDs(xmlDocPtr doc, xmlNodePtr cur) ;

/* Init World Structure and fill it with XML config  */
void CGI_Init_World() {
    
	int list_init = 0 ;
    xmlDocPtr doc;
    xmlNodePtr cur;
    graphPtr cur_graph,ptr = NULL ;


    /* COMPAT: Do not generate nodes for formatting spaces */
    xmlKeepBlanksDefault(0);
    
    /*
     * build an XML tree from a the file;
     */
    doc = xmlParseFile(CGIWorld.configfile);
    if (doc == NULL) {
		_error(-1,"Cant (xml)Parse : %s",CGIWorld.configfile);
	}

    /*
     * Check the document is of the right kind
     */
    
    cur = xmlDocGetRootElement(doc);
    if (cur == NULL) {
		xmlFreeDoc(doc);
        _error(-1,"empty document\n");
    }

    /* Set some default values */
	CGIWorld.step = RRDSTEP ;
    CGIWorld.nbGraph = 0; 	 
	CGIWorld.outputdir = NULL ;
	CGIWorld.includedir = NULL ;
	CGIWorld.cssfile = NULL ;
    CGIWorld.width=DEFAULT_GRAPH_WIDTH ; 	 
    CGIWorld.height=DEFAULT_GRAPH_HEIGHT ; 	 
	CGIWorld.default_view = NULL ;

    /* Second level is a list of Job, but be laxist */
    cur = cur->xmlChildrenNode;
    
    while (cur != NULL) {

	/* Parse Graph  */
	if ((!xmlStrcmp(cur->name, (const xmlChar *) "img_output_dir")))
	    CGIWorld.outputdir = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),NORMAL,NOBLANK);

	else if ((!xmlStrcmp(cur->name, (const xmlChar *) "img_include_dir")))
	    CGIWorld.includedir = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),NORMAL,NOBLANK);
	
	else if ((!xmlStrcmp(cur->name, (const xmlChar *) "img_format"))) 
	    CGIWorld.imgformat = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),UPPERCASE,NOBLANK);
	
	else if ((!xmlStrcmp(cur->name, (const xmlChar *) "cgi_default_view"))) 
	    CGIWorld.default_view = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),UPPERCASE,NOBLANK);
	
	else if ((!xmlStrcmp(cur->name, (const xmlChar *) "img_width"))) 
	    CGIWorld.width = getintfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
	
	else if ((!xmlStrcmp(cur->name, (const xmlChar *) "img_height")))
	    CGIWorld.height = getintfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
	
	else if ((!xmlStrcmp(cur->name, (const xmlChar *) "css_file")))
	    CGIWorld.cssfile = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),NORMAL,NOBLANK);

	else if ((!xmlStrcmp(cur->name, (const xmlChar *) "graph"))) 
	{
	    
	    if (CGIWorld.nbGraph >= MAXGRAPH) break;

	    /* Call the parsegraph routine */
	    cur_graph = parseGraph(doc, cur);
	    
		/* Link the ret structure to the begining of the chained list */
		if (list_init == 0 ) { 
    		CGIWorld.graph = cur_graph; 
			ptr = cur_graph ;
			list_init++ ; 
		}
		
		CGIWorld.nbGraph++;
		ptr->next = cur_graph ;
		ptr = ptr->next ;
	}

	cur = cur->next;
    }

	if (CGIWorld.default_view == NULL)
		CGIWorld.default_view = xstrdup ("day") ;
	else {
		if (strcmp(CGIWorld.default_view,"day") != 0 
			&& strcmp(CGIWorld.default_view,"week") != 0 
			&& strcmp(CGIWorld.default_view,"month") != 0 
			&& strcmp(CGIWorld.default_view,"year") != 0 ) 
		{
			free(CGIWorld.default_view); 
			CGIWorld.default_view = xstrdup("day") ;
		}
	}

    if (CGIWorld.outputdir == NULL)
	    CGIWorld.outputdir = xstrdup("/tmp/");
    
	if (CGIWorld.includedir == NULL)
	    CGIWorld.includedir = xstrdup("/tmp/");
	
	if (CGIWorld.cssfile == NULL)
	    CGIWorld.cssfile = xstrdup("/happystats.css");
	
	if (CGIWorld.imgformat == NULL)
	    CGIWorld.imgformat=xstrdup("PNG"); 

    /* Clean up everything else before quitting. */
	xmlCleanupParser();

}

graphPtr parseGraph(xmlDocPtr doc, xmlNodePtr cur) {
    
   int list_init = 0 ;
   graphPtr ret ;
   graphDsPtr cur_graphds , ptr = NULL ;

   /*  allocate the struct */
    ret = (graphPtr) malloc(sizeof(graph));
	checkmalloc(ret);
    memset(ret, 0, sizeof(graph));

    /* We don't care what the top level element name is */
    cur = cur->xmlChildrenNode;
   
    /* Set some default values */
    ret->nbDs=0;

	ret->time_unit = NULL ;


    /* Search for our Options  */
    while (cur != NULL) {

        if ((!xmlStrcmp(cur->name, (const xmlChar *) "ds"))) {
	    
	    if (ret->nbDs >= MAXDSPERGRAPH) break;

	    /* Parse the ds */
	    
	    cur_graphds = parseGraphDs(doc,cur) ; 

	    if (cur_graphds != NULL ) { 

		/* Link the ret structure to the begining of the chained list */
		if (list_init == 0 ) { 
    			ret->ds=cur_graphds; 
			list_init++ ; 
			ptr = cur_graphds ;
		}
		
		ret->nbDs++;

		ptr->next = cur_graphds ;
		
		ptr = ptr->next ;
		
	    } 
	}
	
	
	else if ((!xmlStrcmp(cur->name, (const xmlChar *) "vertical_label"))) 
	    ret->vertical_label = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),NORMAL,FULL);
	
	else if ((!xmlStrcmp(cur->name, (const xmlChar *) "title"))) 
	    ret->title = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),NORMAL,FULL);
	
	else if ((!xmlStrcmp(cur->name, (const xmlChar *) "time_unit")))
	    ret->time_unit = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),NORMAL,NOBLANK);

	cur = cur->next;
    }
	
	if (ret->time_unit == NULL)
		ret->time_unit = xstrdup ("min") ;
	else {
		if (strcmp(ret->time_unit,"sec") != 0 
			&& strcmp(ret->time_unit,"min") != 0 
			&& strcmp(ret->time_unit,"hour") != 0 )
		{
			free(ret->time_unit); 
			ret->time_unit = xstrdup("min") ;
		}
	}


    /* Check for minimal values */
    if (ret->nbDs == 0) { 
	 freegraph(ret);
	 return(NULL) ;
    } 
	

    return(ret);
}



graphDsPtr parseGraphDs(xmlDocPtr doc, xmlNodePtr cur) {
   
    graphDsPtr ret = NULL;

   /*  allocate the struct */
    ret = (graphDsPtr) malloc(sizeof(graphDs));
    if (ret == NULL) {
        fprintf(stderr,"out of memory\n");
	return(NULL);
    }
    memset(ret, 0, sizeof(graphDs));
    
    /* We don't care what the top level element name is */
    cur = cur->xmlChildrenNode;
    
    while (cur != NULL) {
	
	    if ((!xmlStrcmp(cur->name, (const xmlChar *) "legend"))) 
	    	ret->legend = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),NORMAL,FULL);  
	    
	    else if ((!xmlStrcmp(cur->name, (const xmlChar *) "rrdfile"))) 
	    	ret->rrdfile = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),NORMAL,NOBLANK);
	    
	    else if ((!xmlStrcmp(cur->name, (const xmlChar *) "type"))) 
	    	ret->type = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),UPPERCASE,NOBLANK);
	    
	    else if ((!xmlStrcmp(cur->name, (const xmlChar *) "color"))) 
	    	ret->color = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),NORMAL,NOBLANK);  
	    
	    else if ((!xmlStrcmp(cur->name, (const xmlChar *) "dsname"))) 
	    	ret->dsname = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),LOWERCASE,NOBLANK);  
	    
	    else if ((!xmlStrcmp(cur->name, (const xmlChar *) "short_legend"))) 
	    	ret->short_legend = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),NORMAL,NOBLANK);  

	    cur = cur->next;
    }

	if ( ret->type == NULL )
		ret->type = xstrdup("LINE") ;

    if (ret->rrdfile == NULL)   {
	    freegraphds(ret);
    	    return(NULL);

    } else if (ret->dsname == NULL )  {
	    freegraphds(ret);
    	    return(NULL);
	    
    } else if ( ret->color == NULL ) { 
	    freegraphds(ret);
    	    return(NULL); 
	} 
	else if ( (strcmp(ret->type,"AREA")!= 0 ) && (strcmp(ret->type,"LINE") != 0) ){ 
	    freegraphds(ret);
    	    return(NULL); 
	}
    else if ( ret->legend == NULL  ){ 
	    ret->legend=(char *)malloc(sizeof(char)*((strlen(ret->dsname) + 1)));
	    sprintf(ret->legend,"%s",ret->dsname);
    }
    else if ( ret->short_legend == NULL  ){ 
	    ret->short_legend=(char *)malloc(sizeof(char)*(strlen(ret->dsname) + strlen("/min") + 1) );
	    sprintf(ret->short_legend,"%s/min",ret->dsname);
    }
    return (ret) ;

	
}

/*
 *	PRINT 
 */

/* Print World Structure */
void CGI_Print_World () 
{

	unsigned int i , j ;
	graphPtr curgraph ;
	graphDsPtr curds ;

	curgraph = CGIWorld.graph ;
	
	for (i=0;i<CGIWorld.nbGraph;i++) { 
		
		printf("<!-- Width: %d Height: %d -->\n",CGIWorld.width,CGIWorld.height);
		printf("<!-- Title : %s Vertical_Label: %s Imgformat : %s -->\n",curgraph->title,curgraph->vertical_label,CGIWorld.imgformat);
		
		curds = curgraph->ds ;

		for (j=0;j<curgraph->nbDs;j++) { 
			printgraphds(curds);
			curds = curds->next ; 
		}

		curgraph = curgraph->next ;

	}


} 

void printgraphds (graphDsPtr ds) {
		
	printf("<!-- dsname : %s rrdfile : %s type : %s -->\n",
		ds->dsname,
		ds->rrdfile,
		ds->type );
	
	printf("<!-- Color : %s Legend: %s Short Legend : %s -->\n",
		ds->color,
		ds->legend,
		ds->short_legend );
}


/*
 * FREE
 */


/*  Free  structure */
void CGI_Free_World () 
{
	u_short i ;

	graphPtr cur_graph ;

	free(CGIWorld.outputdir);
	free(CGIWorld.includedir);
	free(CGIWorld.cssfile);
	free(CGIWorld.configfile);
	free(CGIWorld.imgformat) ;
	free(CGIWorld.default_view);

	cur_graph = CGIWorld.graph ;

	for (i = 0 ; i < CGIWorld.nbGraph ;i++) { 
		freegraph(cur_graph);
		cur_graph = cur_graph->next ;
	}

}

char *freegraph(graphPtr graph) { 

	int i; 
	graphDsPtr cur ;
	
	if (graph->vertical_label) free(graph->vertical_label) ;
	if (graph->title) free(graph->title) ;
	
	cur=graph->ds ;
	for (i=0;i<graph->nbDs;i++) { 
		freegraphds(cur);
		cur = cur->next ;
	}
	
	free(graph);

	return(NULL) ;
}

char *freegraphds(graphDsPtr graphDs) { 

	if (graphDs->short_legend) free(graphDs->short_legend) ;
	if (graphDs->legend) free(graphDs->legend) ;
	if (graphDs->color) free(graphDs->color) ;
	if (graphDs->type) free(graphDs->type) ;
	if (graphDs->dsname) free(graphDs->dsname) ;
	if (graphDs->rrdfile) free(graphDs->rrdfile) ;
	
	free(graphDs);
	return(NULL) ;
}
