/********************************************************************************/
/*                                                                              */
/*     Happystats $Id: common.c,v 1.2 2004/01/25 22:38:20 ofredj Exp $		*/
/*                                                                              */
/*     This program is free software; you can redistribute it and/or modify     */
/*     it under the terms of the GNU General Public License as published by     */
/*     the Free Software Foundation; either version 2 of the License, or        */
/*     (at your option) any later version.                                      */
/*                                                                              */
/*     This program is distributed in the hope that it will be useful,          */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            */
/*     GNU General Public License for more details.                             */
/*                                                                              */
/*     You should have received a copy of the GNU General Public License        */
/*     along with this program; if not, write to the Free Software              */
/*     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
/*                                                                              */
/********************************************************************************/

/* Includes */
#include "happystats.h"
  
/* Output debug */
void _debug(int display, int err, char *fmt, ...) {
    va_list va;

    if (!display) return;
    va_start(va, fmt);
    fprintf(stderr, "D: ");
    vfprintf(stderr, fmt, va);
	if (err) {
    	fprintf(stderr, ": %s", strerror(err));
	}
    fprintf(stderr, "\n");
    va_end(va);
}

/* Output error and Exit */
void _error(int err, char *fmt, ...) {
    va_list va;
    va_start(va, fmt);
    fprintf(stderr, "E: ");
    vfprintf(stderr, fmt, va);
	if (err) {
    	fprintf(stderr, ": %s", strerror(err));
	}
    fprintf(stderr, "\n");
    va_end(va);
    exit(err);
}

/* Itoa function - convert an integer to a string pointed by ptr */
char *itoa(int i) {
  static char buf[MAX_INT_STR];
  if (sprintf(buf, "%d", i)>0) return buf;
  _debug(1, errno, "Can't convert %d to integer", i);
  return NULL;
}

/* Check malloc */
void checkmalloc(void *buffer) {
  if (buffer) return;
  _error(errno, "Can't allocate buffer");
}

/* The strdup function with return test included */
char *xstrdup (const char *str) {
  char *ret;
  if ((ret=strdup(str))) return ret;
  _error(errno, "Can't duplicate buffer '%s' ", str);
  return NULL;
}

/* 
 * Returns the requested formatted string from an xmlChar field .
 */
char *getcharfield (xmlChar *field,enum Fmt_Case FCASE,enum Fmt_Str FSTR) { 

	unsigned short int i , maxchar;
	char *ret ;

	ret = (char *)malloc((xmlStrlen(field)+1)*sizeof(char)) ;
	
	maxchar = xmlStrlen(field) ;

	
	if (field)
	{
		if (FSTR == NOBLANK) {
			while(*field==' ')
				field++ ; 
		} 

		for (i=0;i < maxchar ;i++) { 
		
			if ((FSTR == NOBLANK) && (field[i]==' '))
				break;

			switch (FCASE) { 
				case LOWERCASE:
					ret[i]=tolower(field[i]); 
					break;
				case UPPERCASE:
					ret[i]=toupper(field[i]); 
					break;
				default:
					ret[i]=field[i]; 
					break;
			}
		}

		ret[i] = '\0' ;
		return(ret) ;

	} else
		return (NULL) ;			
}

/* 
 *	Return the integer value of an xmlChar field 
 */
int getintfield (xmlChar *field) { 
		
	long int i = 0;
	char *ret = (char *)malloc((xmlStrlen(field))*sizeof(char)) ;
	
	if (field)
	{
		while(*field==' ') { field++ ; }

		for (i=0;field[i] && (field[i]!=' ');i++)
		{
			ret[i]=field[i]; 
		}

		ret[i] = '\0' ;
		i=atoi(ret);
		free(ret);

		return(i);

	} else {
		return (0) ;
	}
}

