/* Includes */
#include "happystats.h"

/* Local Structs */
static struct option long_options[] =  {
    {"configfile", 1, 0, 'c'},
    {"logformat", 1, 0, 'f'},
    {"year", 1, 0, 'y'},
    {"help",      0, 0, 'h' },
    {"verbose",   0, 0, 'v' },
    {"debug",   0, 0, 'd' },
    {"version",   0, 0, 'V' },
    {0, 0, 0, 0}
};

static char helptext[][255] = {
    {"Config file"},
    {"Log format"},
    {"Set the year (4 digits)"},
    {"Show this help screen"},
    {"Run in verbose mode"},
    {"Run in debug mode"},
    {"Print version number"},
    { "" }
};

/* Usage */
void usage() {
  
  int i ;    
     
  printf("Happystats version %s\n", VERSION);
  printf("Usage: happystats [options] logfile\n");
  for(i=0; strcmp(helptext[i], ""); i++) {
	  printf(" -%c, --%-12s \t%s\n", long_options[i].val ,
			  long_options[i].name, helptext[i]);
  }
  printf("\n");
  exit(0);
}

/* Parse command line */
void GetOpt(int argc,char **argv) {
  int i, c;
  int option_index = 0;


  /* Set default values */
  World.verbose=0;
  World.debug=0;
  World.year=0;


  while (1) {
    c = getopt_long (argc, argv,"y:c:f:vdVh",long_options, &option_index);
    if (c == -1 || c == EOF)
      break;

    switch (c) {
    	case 'y':
				World.year = atoi(optarg);
      			break;
    	case 'c':
				World.configfile = xstrdup(optarg);
      			break;
    	case 'f':
				World.logformat = xstrdup(optarg);
      			break;
    	case 'v':
      		World.verbose=1;
      		break;
    	case 'd':
      		World.debug=1;
      		World.verbose=1;
      		break;
    	case 'V':
      		printf("Happystats version %s\n", VERSION);
      		exit(0);
      		break;
    	case 'h':
    	case ':':
    	case '?':
     		printf("Happystats version %s\n\n", VERSION);
      		printf("Usage: %s [options] logfile\n", argv[0]);
      		for(i=0; strcmp(helptext[i], ""); i++) {
				printf(" -%c, --%-12s \t%s\n", long_options[i].val ,long_options[i].name, helptext[i]);
      		}
      		printf("\n");
      		exit(0);
      		break;
    }
  }

  if (optind == argc) {
    World.logfile = xstrdup("stdin");
  } else {
	World.logfile = xstrdup(argv[optind]);
  }
  
  /* Check for configfile */
  if (World.configfile == NULL)
  {
	/* configfile not passed as argument check if an default happystats.conf exists */
    if (existfile(DEFAULT_CONFIG_FILE,"r") == 0) 
  		World.configfile=xstrdup(DEFAULT_CONFIG_FILE);
	else 
		_error(1,"No config file found");
  
  } 
  else if (existfile(World.configfile,"r") == 1 )  
		_error(1,"Config file %s not found",World.configfile) ;

  /* Set log format  */
  if (World.logformat == NULL)
		  World.logformat = xstrdup("syslog");


}

