/********************************************************************************/
/*                                                                              */
/*     Happystats $Id: happystats.h,v 1.11 2004/02/22 12:26:26 ofredj Exp $	*/
/*                                                                              */
/*     This program is free software; you can redistribute it and/or modify     */
/*     it under the terms of the GNU General Public License as published by     */
/*     the Free Software Foundation; either version 2 of the License, or        */
/*     (at your option) any later version.                                      */
/*                                                                              */
/*     This program is distributed in the hope that it will be useful,          */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            */
/*     GNU General Public License for more details.                             */
/*                                                                              */
/*     You should have received a copy of the GNU General Public License        */
/*     along with this program; if not, write to the Free Software              */
/*     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
/*                                                                              */
/********************************************************************************/

/* 
 * Defines
 * -- for very big happystats user you'll have to increase that limits 
 */
#define VERSION	"0.5.0"

#define MAX_INT_STR 128 
#define OVECCOUNT 60    /* should be a multiple of 3 */
#define RRDSTEP 60    /* Minimum Timestamp increment to be Updatable  */
#define POINTS_PER_SAMPLE 3
#define _XOPEN_SOURCE 

#ifdef __linux__
#define DEFAULT_CONFIG_FILE "/etc/happystats.conf"
#define DEFAULT_MAILCMD	"/bin/mail"
#else 
#define DEFAULT_CONFIG_FILE "/usr/local/etc/happystats.conf"
#endif
#define DEFAULT_MAILHOST "127.0.0.1"
#define DEFAULT_MAILHOSTPORT 25
#define DEFAULT_ALERT_INTERVAL 300 /* 5 mins between each alerts at least  */
/* Includes */
#ifdef __linux__
#include <errno.h>
#define _GNU_SOURCE
#include <bits/posix2_lim.h>
#include <sys/types.h>
#else
#if __FreeBSD_version > 500000
#include <sys/limits.h>
#else
/* #include <machine/limits.h> */
#include <limits.h>
#endif
#endif
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <time.h>

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <pcre.h>
#include <rrd.h>
#ifdef __Darwin__
#include <gnugetopt/getopt.h> /* Require gnugetopt under FreeBSD */
#else
#include "getopt.h" /* Require gnugetopt under FreeBSD */
#endif 

/* 
 *
 * Structures 
 * 
 */

struct filehandle {
  FILE *fd ;
  char *line ;
  u_short position ; 
  u_short filedesc ;
};

/* Struct containing the parsed logline */
struct plogline {
  u_long timestamp;
  char *program;
  char *hostname;
  char *action;
};

/* Struct DS */
typedef struct ds {
    char *dsname;
    u_short dscount;
    struct ds *next ;
} ds, *dsPtr;

typedef struct Gds {
    char *dsname_ptr;
    u_short *dscount_ptr;
    struct Gds *next ;
} Gds, *GdsPtr;



typedef struct rrdDB {
    char *rrdfile ;
    u_short nbDs;
    dsPtr ds;
    struct rrdDB *next ;
} rrdDB, *rrdDBPtr;

/* Rules structure */
typedef struct rules {
    u_short brk ;		/* bool -- to break or not to break ? */
	char *dsname ;		/* if rule match update this DS */
    pcre *re_host ;		/* regexp to match the host */
    pcre *re_program ;	/* regexp to match the program */
    pcre *re_action	;	/* regexp to match the action (rest of the logline after program) */
    struct rules *next ;/* ptr to next struct */
} rules, *rulesPtr;

/* Alerts stucture */
typedef struct alert {
	char *dsname ;		/* if rule match update this DS */
	char *mailrcpt ;		/* to you send the mail ? 	*/
	char *mailsubject ;		/* Subject of the mail 		*/
	u_long treshold ; 	/* Value of the treshold  	*/
	u_short nb_hit_treshold ; 	/* Nombre d  	*/
	u_long last_alert ;		/* timestamp de la derniere alerte envoye */
    pcre *re_host ;		/* regexp to match the host */
    pcre *re_program ;	/* regexp to match the program */
    pcre *re_action	;	/* regexp to match the action (rest of the logline after program) */
	struct alert *next ;	/*	next element  */
} alert, *alertPtr ;

/* 
 * World Global Struct 
 */
struct world {
	
	/* Options */
	u_short verbose; 	/* print verbose infos */
	u_short debug;		/* print debug + verbose infos */
	u_long start ;		/* timestamp of database creation optionnal (default now - 10s)*/
	u_short step ;		/* update intervall */
	u_short width ;		/* graph width -- used for rrddb creation */
	char *configfile ;  /* full path to the configfile */
	char *logfile ;		/* full path to the logfile */
	char *logformat ;	/* format of the logfile */
	u_short year ;
	
	/* Current Timestamp */
    long timestamp;  /* Current log timespamp  */
    long lastupdate; /* Last Update timestamp  */
    
	/* Rules linked list*/
	u_short nbRules;
    rulesPtr rules ; 
    
	/* RRD DB linked list*/
	u_short nbrrdDB;
    rrdDBPtr rrdDB ;

	/* Alerts */
	struct _alerts {
		u_short nbAlertTreshold ; 	/* number of alerts	*/
		alertPtr AlertTreshold ;	/* linked list of alerts */
		
		u_short nbAlertRules ; 	/* number of alerts	*/
		alertPtr AlertRules ;	/* linked list of alerts */
		
		/*char *mailcmd ;	 		* path to the mail cmd */
		char *mailhost ;			/* Smart Host */
		u_short mailhostport ;		/* Smart Host Port */

		u_short interval ;			/* minimum interval between alerts */
	} Alerts ;

	/* G(lobal)DS linked list (view on RRDdb's DS liked lists) */
	u_short nbGDS ;
	GdsPtr	GDS ;

	/* A pointer to the regexp used by the Parser */
	pcre *Parser_Rule ;
	/* A callback to the Parser */
	int (*Parser_Callback)(char *logline) ;

	/* Structure containing parsed-line  */
	struct plogline parsedline ;
	
	/* Structure containing file-handler  */
	struct filehandle fh ;


} World ;


/* 
 *
 * Prototypes 
 * ----------
 */


/* 
 * Xmlparser functions 
 */
void Get_XmlConf() ;
void Build_Global_DS_List () ;
void Print_World() ;
void Free_World () ;

/* 
 * PCRE functions 
 */
pcre *regcomp(char *pattern);
int regmatch (char *string, pcre *re);
char *fetch_substrings (char *logline, int ovector[], int i);
void Rules_Engine () ;

/* 
 * Alerts Functions 
 */
void Alert_Rules_Engine ();
void Alerts_Treshold_Engine();

/*
 * RRD functions 
 */
int getlastupdate (char *filename);
int update_rrd (rrdDBPtr DB) ;
int create_rrd (rrdDBPtr DB) ;
void Reset_Counters ();


/* 
 * Date functions
 */
u_short monthatoi (char *month); /* */
u_long date2timestamp (int day,int month,int year,int hour,int minute,int second);

/*
 * Common functions 
 */
void _debug(int display, int err, char *fmt, ...);
void _error(int err, char *fmt, ...);
char *xstrdup (const char *str); /* strdup wrapper */
char *itoa(int i); /* Convert int to char */
void checkmalloc(void *buffer); /* check malloc return */


/* File */
int Open_fh(); /* Open filehandler */
int Read_fh();
int Close_fh(); /* Close filehandler*/
int existfile(char *filename,const char *attr) ;

/* getopt functions */
void GetOpt(int argc,char **argv) ; /* Parse command line */
void usage() ; /* Print Usage */

/* 
 * Enums 
 */
enum Fmt_Case { NORMAL,UPPERCASE,LOWERCASE } ;
enum Fmt_Str { FULL,NOBLANK } ;

/* Misc */
char *getcharfield (xmlChar *field,enum Fmt_Case FCASE,enum Fmt_Str FSTR) ;
int getintfield (xmlChar *field) ;

/* Syslog functions */
void Compile_Syslog_Rule() ; /* Compile Syslog Rule -- if fails exit */
int Parse_Syslog(char *logline); /* Split a syslog line into pieces (fill struct parsedline ) */

/* IMS functions */
void Compile_IMS_Rule() ; /* Compile Syslog Rule -- if fails exit */
int Parse_IMS(char *logline); /* Split a syslog line into pieces (fill struct parsedline ) */

/* Exim functions */
void Compile_Exim_Rule() ; /* Compile Syslog Rule -- if fails exit */
int Parse_Exim(char *logline); /* Split a syslog line into pieces (fill struct parsedline ) */

/* CLF functions */
void Compile_CLF_Rule() ; /* Compile Syslog Rule -- if fails exit */
int Parse_CLF(char *logline); /* Split a syslog line into pieces (fill struct parsedline ) */

int Happy_sendmail(char *subject,char *message,char *rcpt);

/* 
 * Global Vars 
 */
extern int errno;
