/********************************************************************************/
/*                                                                              */
/*     Happystats $Id: main.c,v 1.4 2004/02/22 12:26:26 ofredj Exp $		*/
/*                                                                              */
/*     This program is free software; you can redistribute it and/or modify     */
/*     it under the terms of the GNU General Public License as published by     */
/*     the Free Software Foundation; either version 2 of the License, or        */
/*     (at your option) any later version.                                      */
/*                                                                              */
/*     This program is distributed in the hope that it will be useful,          */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            */
/*     GNU General Public License for more details.                             */
/*                                                                              */
/*     You should have received a copy of the GNU General Public License        */
/*     along with this program; if not, write to the Free Software              */
/*     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
/*                                                                              */
/********************************************************************************/

/* Includes */
#include "happystats.h"

/* The HappyStats main section  8*/
int main(int argc,char **argv) {
  
  /* Variables */
  u_short ret = 0 , i = 0 ;
  u_short done;
  
  /* Structures */
  rrdDBPtr DB = NULL ;

  /* Get option line */
  GetOpt(argc, argv);

  /* Get (XML)conf */
  Get_XmlConf();

  /* check if we found enought data to continue */
  if (World.nbrrdDB == 0 || World.nbRules == 0 )
		_error(0,"No rules nor Ds found in config file : %s",World.configfile);
  
  /* Global View on each RRDdb chained list */
  Build_Global_DS_List ();
  
  /* Print conf */
  if (World.verbose)
    	Print_World();


  World.lastupdate = 0 ;

  DB = World.rrdDB ;
  for (i=0; i < World.nbrrdDB ;i++) {

  	/* Check if rrd database exists, restore it from configfile if not found */
  	if (existfile(DB->rrdfile,"r+") != 0 ) {
		create_rrd(DB);
  	} 
  	
	/*  World.lastupdate is the most recent updated rrd database */
	if ((getlastupdate(DB->rrdfile)) > World.lastupdate) {
		World.lastupdate = getlastupdate(DB->rrdfile); 
	}

	DB = DB->next ;
  } 

  World.timestamp = World.lastupdate ;

  /* Compile Parser Rule And Register CallBack */
  if (strcasecmp(World.logformat,"syslog") == 0 ) {
	 Compile_Syslog_Rule();
	 World.Parser_Callback = Parse_Syslog ;
  } else if (strcasecmp(World.logformat,"ims") == 0 ) {
	Compile_IMS_Rule();
	World.Parser_Callback = Parse_IMS ;
  } else if (strcasecmp(World.logformat,"exim") == 0 ) {
	Compile_Exim_Rule();
	World.Parser_Callback = Parse_Exim ;
  } else if (strcasecmp(World.logformat,"clf") == 0 ) {
	Compile_CLF_Rule();
	World.Parser_Callback = Parse_CLF ;
  } else { 
	_error(0,"Could not find the requested parser .. exiting \n"); 
  }

  /* Init the counters */
  Reset_Counters();

  /* Open the logfile */
  if (strcmp(World.logfile,"stdin") != 0 )
  {
	ret = Open_fh();
  	if (ret == 1)
  	  usage();
  }
  
  /* malloc some space for the line(s)*/
  World.fh.line = (char *)malloc(_POSIX2_LINE_MAX*sizeof(char)) ;
  checkmalloc(World.fh.line);

  /* Here we go ! */
  for (;;) {

	/* Read line from fd/stdin */
	done = Read_fh();
	
	/* If done break */
	if(done) { break ; }
	
	if (World.debug)
		_debug(2,0,"%s",World.fh.line);
  
    /* Split logline */ 
	if (World.Parser_Callback(World.fh.line) == 0) 
	{
			
	  if (World.debug)
	  	_debug(2,0,"\thost: %s\n\tprogram: %s\n\taction: %s\n",
				World.parsedline.hostname,World.parsedline.program,World.parsedline.action);

	  if (World.parsedline.timestamp >= World.timestamp) 
	  {
		
	    /* Set the timestamp */
	    World.timestamp = World.parsedline.timestamp;
		
		/*  The Rules Engine */
		Rules_Engine();

		/* The Alert Rule Engine */
		Alert_Rules_Engine();
	
		/* Update the graphs if there is more than RRDSTEP seconds   */
		if ( World.timestamp > ( World.lastupdate + World.step )) 
		{
			/* Update time */
			World.lastupdate = World.timestamp ;
			
			DB = World.rrdDB ;
			for ( i = 0 ; i < World.nbrrdDB ; i++ ) 
			{
				/*Update rrd database */
				update_rrd(DB);
			
				DB = DB->next ;
        	}

			/* Check for Alerts Treshold */
			Alerts_Treshold_Engine();
			

			/* Reset the counter  */
  			Reset_Counters();
		}
		
	  }
		
	  /* Free P_LINE struct members */
	  free(World.parsedline.hostname)  ;
	  free(World.parsedline.program)  ;
	  free(World.parsedline.action)  ;
        	
	} 



  }

  /* Close file  */
  if (strcmp(World.logfile,"stdin") != 0 ) {
  	ret = Close_fh() ;
  } else { 
		  ret= 0 ; 
  }
  
  if (World.verbose == 1) {
  	printf("Everthing run's fine and you're in a happy small world\n");
  	printf("That's All folk !\n");
  }

  /* Free XMLCONF */
  Free_World() ;

  /* Happy End ! */
  exit(ret) ;
}
