/********************************************************************************/
/*                                                                              */
/*     Happystats $Id: pcre.c,v 1.3 2004/02/08 17:18:11 ofredj Exp $		*/
/*                                                                              */
/*     This program is free software; you can redistribute it and/or modify     */
/*     it under the terms of the GNU General Public License as published by     */
/*     the Free Software Foundation; either version 2 of the License, or        */
/*     (at your option) any later version.                                      */
/*                                                                              */
/*     This program is distributed in the hope that it will be useful,          */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            */
/*     GNU General Public License for more details.                             */
/*                                                                              */
/*     You should have received a copy of the GNU General Public License        */
/*     along with this program; if not, write to the Free Software              */
/*     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
/*                                                                              */
/********************************************************************************/

/* Includes */
#include "happystats.h"

/* Fetch a substring from an regexp output */
char *fetch_substrings (char *logline, int ovector[], int i) {
  int substring_length ;
  int stringcount = _POSIX2_LINE_MAX ;
  char *substring_start ;
  char *substring;

  substring_start = logline + ovector[2*i];
  substring_length = ovector[2*i+1] - ovector[2*i] + 1 ;

  substring=(char *)malloc(substring_length*sizeof(char));
  checkmalloc(substring);

  pcre_copy_substring(logline,ovector,stringcount,i,substring,substring_length);

  /* WARNING we've  malloc the substring Don't forget to FREE IT !!!!*/
  return (substring);
}

/* check if a given string match a given regexp  */
int regmatch (char *string, pcre *re) {
  int rc;
  int ovector[OVECCOUNT];

  /* Do the match test */
  rc = pcre_exec(
		 re,                    /* the compiled pattern */
		 NULL,                  /* we didn't study the pattern */
		 string,                /* the subject string */
		 (int)strlen(string)+1,   /* the length of the subject */
		 0,                     /* start at offset 0 in the subject */
		 0,                     /* default options */
		 ovector,               /* vector for substring information */
		 OVECCOUNT);            /* number of elements in the vector */
  
  return (rc);
}

/* Compile a pattern , return a malloced pcre *re */
pcre *regcomp(char *pattern) {
  pcre *re;
  const char *error;
  int erroffset;

  if (!pattern)
	  return(NULL);

  re = pcre_compile(
		    pattern,        /* the pattern */
		    PCRE_EXTENDED|PCRE_DOTALL|PCRE_MULTILINE,  /* default options */
		    &error,         /* for error message */
		    &erroffset,     /* for error offset */
		    NULL);
  if (re == NULL)
	  _debug(2,1,"bad rule : %s ",pattern);

  return(re);
}


/* Match a parsed logline against registered rules 
 * Update the dscount if match */
void Rules_Engine () { 

	u_short  i, j, rc ;	
	rulesPtr cur = NULL ;
	struct Gds *DS ;

	cur = World.rules ;
    	
	for (i = 0; i < World.nbRules; i++) {  
		rc = 0 ;
		
		if (cur->re_host == NULL ) {
			rc++; /* No regexp so match in any case */ 
		} else { 
			if (regmatch(World.parsedline.hostname,cur->re_host) > 0 ) { 
				rc++;
			} else { 
					cur = cur->next ;
					continue ; 
			}
				 
		}
		
		if (cur->re_program == NULL)  {
			rc++; 
		} else { 
			if (regmatch(World.parsedline.program,cur->re_program) > 0) {
				rc++;
			} else { 
					cur = cur->next ;
					continue ; 
			}

		}
		

		if (cur->re_action == NULL )  {
			rc++; 
		} else { 
			if (regmatch(World.parsedline.action,cur->re_action) > 0) {
				rc++;
			} else { 
					cur = cur->next ;
					continue ; 
			}
		}

			

		/* The 3 regexp match update the dscount */
		if (rc == 3) { 
			
			/* Update corresponding DS */
			DS = World.GDS ;
			for (j = 0 ; j < World.nbGDS  ; j++ ) 
			{ 
				if (strcmp(DS->dsname_ptr,cur->dsname) == 0) {
					*DS->dscount_ptr = *DS->dscount_ptr + 1 ; 
					break ;
				}
				DS = DS->next ;
			}
				

			/* If we work in single rule matching uncomment the following line */
			if (cur->brk == 0) { return ; }
		}
		
		cur = cur->next ;

	}

}

