/********************************************************************************/
/*                                                                              */
/*     Happystats $Id: rrd.c,v 1.4 2004/02/07 19:00:43 ofredj Exp $ 	*/
/*                                                                              */
/*     This program is free software; you can redistribute it and/or modify     */
/*     it under the terms of the GNU General Public License as published by     */
/*     the Free Software Foundation; either version 2 of the License, or        */
/*     (at your option) any later version.                                      */
/*                                                                              */
/*     This program is distributed in the hope that it will be useful,          */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            */
/*     GNU General Public License for more details.                             */
/*                                                                              */
/*     You should have received a copy of the GNU General Public License        */
/*     along with this program; if not, write to the Free Software              */
/*     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
/*                                                                              */
/********************************************************************************/


/* Includes */
#include "happystats.h"


/* get the last update time i of an rrdfile */
int getlastupdate (char *filename) {

  int rc;
  char *rrdcmd[] = {  
	"last",
	NULL,
	NULL 	
  };

  rrdcmd[1] = (char *)malloc(sizeof(char)*(strlen(filename)+1));
  checkmalloc(rrdcmd[1]);

  rrdcmd[1] = xstrdup(filename) ;

  optind = 0;
  rrd_clear_error();
  /* Get the last update timestamp */
  rc=rrd_last(2,rrdcmd);
  
  if (rc == -1)
    _debug(1, errno, "Can't get last update time from RRD '%s': %s", filename, rrd_get_error());

  /* Free Args */
  free(rrdcmd[1]);

  return(rc);
}


/* Format and Update rrd database file */
int update_rrd (rrdDBPtr DB) {
	
  int rc,i ;
  dsPtr ds;

  char *rrdcmd[] = {	
	"update",
	NULL,
	NULL,
	NULL
  };
    

  rrdcmd[1] = (char *)malloc(sizeof(char)*(strlen(DB->rrdfile)+1));
  checkmalloc(rrdcmd[1]);
  strncpy(rrdcmd[1],DB->rrdfile,strlen(DB->rrdfile)+1) ;

  rrdcmd[2] = (char *)malloc(sizeof(char)*(_POSIX2_LINE_MAX));
  checkmalloc(rrdcmd[2]);

  sprintf(rrdcmd[2],"%ld",World.lastupdate);

  ds = DB->ds ;
  for (i = 0; i < DB->nbDs; i++) {
	  sprintf(rrdcmd[2],"%s:%d",rrdcmd[2],ds->dscount); 
	  ds = ds->next ;
  }

  optind = 0;
  rrd_clear_error();
  
  rc=rrd_update(3,rrdcmd) ;
  
  if (World.verbose == 1) { 
  	_debug(1,0,"rrdupdate : %s",rrdcmd[2]);
  }

  if (rc) {
    _debug(1, errno, "Can't update RRD '%s': %s", DB->rrdfile, rrd_get_error());
  }

  free(rrdcmd[1]);
  free(rrdcmd[2]);
  
  /* Return the value of rrdupdate */
  return(rc) ;
}


/* Put ds->dscount struct values to 0 */
void Reset_Counters ()
{ 

    u_short i ;
    GdsPtr cur_ds;

    cur_ds = World.GDS ;
    for (i = 0; i < World.nbGDS; i++) {
   	    *cur_ds->dscount_ptr = 0 ; 
	    cur_ds = cur_ds->next ;
    }
	
}


/* Create an rrd file from an rrdDB structure */
int create_rrd (rrdDBPtr DB) { 

    int i = 0,j ,rc ;
    int  rrdargc = ( DB->nbDs + 12 ) ;
    dsPtr cur_ds;
    char *rrdcmd[64] ;


	int rows = World.width / POINTS_PER_SAMPLE ; 
	int realrows = (int)(rows * 1.1);
	int day_steps = (int)(3600*24 / (World.step * rows)) ;
	int week_steps = day_steps * 7 ;
	int month_steps = week_steps * 5 ;
	int year_steps = month_steps * 12 ;


	/* step = 3600 / World.step ; */
    
    rrdcmd[i] = xstrdup ("create"); 
    
    i++ ; 

    rrdcmd[i] = (char *)malloc((strlen(DB->rrdfile)+1)*sizeof(char));
    checkmalloc(rrdcmd[i]);
    strncpy(rrdcmd[i],DB->rrdfile,strlen(DB->rrdfile)+1);
    i++ ; 

    rrdcmd[i] = xstrdup ("--step"); 
    i++ ; 

    rrdcmd[i] = (char *)malloc(MAX_INT_STR*sizeof(char));
    checkmalloc(rrdcmd[i]);
    sprintf(rrdcmd[i],"%d",World.step);
    i++ ; 

    if ( World.start != 0 ) {
	    rrdcmd[i] = xstrdup ("--start"); 
	    i++ ; 
    	    rrdcmd[i] = (char *)malloc(MAX_INT_STR*sizeof(char));
            checkmalloc(rrdcmd[i]);
            sprintf(rrdcmd[i],"%ld",World.start);
	    i++ ; 
	    rrdargc+=2;
    } 

    cur_ds = DB->ds ;
    for (j = 0; j < DB->nbDs; j++) { 
    
    	    
	    rrdcmd[i] = (char *)malloc(_POSIX2_LINE_MAX*sizeof(char));
    	    
	    checkmalloc(rrdcmd[i]);
	    
        sprintf(rrdcmd[i],"DS:%s:ABSOLUTE:%d:0:U",cur_ds->dsname,(World.step * 2));
	    i++;

	    cur_ds = cur_ds->next ;
    }
    

   	rrdcmd[i] = (char *)malloc(_POSIX2_LINE_MAX*sizeof(char));
   	checkmalloc(rrdcmd[i]);
   	sprintf(rrdcmd[i++],"RRA:AVERAGE:0.5:%d:%d",day_steps,realrows*7);

   	rrdcmd[i] = (char *)malloc(_POSIX2_LINE_MAX*sizeof(char));
   	checkmalloc(rrdcmd[i]);
   	sprintf(rrdcmd[i++],"RRA:AVERAGE:0.5:%d:%d",week_steps,realrows*4);

   	rrdcmd[i] = (char *)malloc(_POSIX2_LINE_MAX*sizeof(char));
   	checkmalloc(rrdcmd[i]);
   	sprintf(rrdcmd[i++],"RRA:AVERAGE:0.5:%d:%d",month_steps,realrows*12);
    	
	rrdcmd[i] = (char *)malloc(_POSIX2_LINE_MAX*sizeof(char));
   	checkmalloc(rrdcmd[i]);
   	sprintf(rrdcmd[i++],"RRA:AVERAGE:0.5:%d:%d",year_steps,realrows*2);
		
   	rrdcmd[i] = (char *)malloc(_POSIX2_LINE_MAX*sizeof(char));
   	checkmalloc(rrdcmd[i]);
   	sprintf(rrdcmd[i++],"RRA:MAX:0.5:%d:%d",day_steps,realrows*7);
    	
	rrdcmd[i] = (char *)malloc(_POSIX2_LINE_MAX*sizeof(char));
   	checkmalloc(rrdcmd[i]);
   	sprintf(rrdcmd[i++],"RRA:MAX:0.5:%d:%d",week_steps,realrows*4);
    	
	rrdcmd[i] = (char *)malloc(_POSIX2_LINE_MAX*sizeof(char));
   	checkmalloc(rrdcmd[i]);
   	sprintf(rrdcmd[i++],"RRA:MAX:0.5:%d:%d",month_steps,realrows*12);
    	
	rrdcmd[i] = (char *)malloc(_POSIX2_LINE_MAX*sizeof(char));
   	checkmalloc(rrdcmd[i]);
   	sprintf(rrdcmd[i++],"RRA:MAX:0.5:%d:%d",year_steps,realrows*2);

    if (World.verbose == 1) {
    	for (j = 0 ; j < rrdargc ; j++) {  
			_debug(1,0,"rrdcreate[%d]: %s",j,rrdcmd[j]); 
		}
    }
  
    /* Restore rrd file from config */
    
    optind = 0;
    rrd_clear_error();

	/* */
    rc = rrd_create(rrdargc, rrdcmd);
    
    if (rc) {
		_debug(1, errno, "Can't create RRD '%s': %s", DB->rrdfile, rrd_get_error());
    } else if (World.verbose) {
    	_debug(1,0,"'%s' created .", DB->rrdfile);
    }
    
    /* Free rrdarg */
    for (j = 0 ; j < rrdargc ; j++) {  
	    free(rrdcmd[j]); 
    }

    /* Return the value of rrd_create */
    return(rc);

} 
