/********************************************************************************/
/*                                                                              */
/*     Happystats $Id: xmlparser.c,v 1.6 2004/02/22 12:26:26 ofredj Exp $	*/
/*                                                                              */
/*     This program is free software; you can redistribute it and/or modify     */
/*     it under the terms of the GNU General Public License as published by     */
/*     the Free Software Foundation; either version 2 of the License, or        */
/*     (at your option) any later version.                                      */
/*                                                                              */
/*     This program is distributed in the hope that it will be useful,          */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            */
/*     GNU General Public License for more details.                             */
/*                                                                              */
/*     You should have received a copy of the GNU General Public License        */
/*     along with this program; if not, write to the Free Software              */
/*     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
/*                                                                              */
/********************************************************************************/

/* Includes */
#include "happystats.h"

/*
 * Local prototypes
 */

rrdDBPtr parse_rrdDB(xmlDocPtr doc, xmlNodePtr cur) ;
rulesPtr parseRule(xmlDocPtr doc, xmlNodePtr cur) ;
void printrrdDB(rrdDBPtr DB) ;
void freesyslogrule(rulesPtr rule) ;
void freerrdDB (rrdDBPtr DB) ;
void free_alert (alertPtr alert);
void free_alert_treshold (alertPtr alert);
void free_alert_rule (alertPtr alert);

/* 
 *
 *	PARSE 
 *
 */

/* */
void Get_XmlConf() {

	xmlDocPtr doc = NULL ;
   	xmlNodePtr cur = NULL , syslog_node = NULL ;
	
	rrdDBPtr DB , DBptr = NULL ;

	rulesPtr cur_rule , ruleptr = NULL ;

   	xmlNodePtr alert_node = NULL , alerts_node = NULL ;
	alertPtr cur_alert = NULL, alertptr = NULL ;
	char *charptr = NULL ;
	char *default_rcpt = NULL ;

   	xmlKeepBlanksDefault(0);
    
   	/*
   	 * build an XML tree from a the file;
   	 */
   	doc = xmlParseFile(World.configfile);
   	if (doc == NULL) { 
		_error(-1,"xmlParseFile error\n") ;
	};

   	/*
  	 * Check the document is of the right kind
   	 */
   	cur = xmlDocGetRootElement(doc);
   	if (cur == NULL) {
		_error(-1,"empty document can't get RootElement\n") ;
	}

   	/* Set some default values */
   	World.nbRules = 0 ;
   	World.nbrrdDB = 0 ;
	World.start = 0 ;
	World.step = RRDSTEP ;
	World.width = 540 ;
	World.Alerts.nbAlertTreshold = 0 ;
	World.Alerts.nbAlertRules = 0 ;
	World.Alerts.mailhost = NULL ;
	World.Alerts.mailhostport = 0 ;
	World.Alerts.interval =  DEFAULT_ALERT_INTERVAL ;
   
	/* Get Down into the XMLTree ! */
   	cur = cur->xmlChildrenNode;
    
   	while (cur != NULL) {
	
       	if ((!xmlStrcmp(cur->name, (const xmlChar *) "rrd")))
		{ /* Parse DS and fill the DS linked list */
			
    		DB = parse_rrdDB (doc,cur) ;
		
			if (DB != NULL) 
			{ 

				if(World.nbrrdDB == 0 ) 
				{
					World.rrdDB = DB ;
					DBptr = DB ;
				}

				DBptr->next = DB ;
				DBptr = DBptr->next ;	
				World.nbrrdDB++ ;
			}
		}
		
		else if ((!xmlStrcasecmp(cur->name, (const xmlChar *) "alerts")))
		{

			alerts_node = cur->xmlChildrenNode ;

			while (alerts_node != NULL) 
			{

				if (xmlStrcmp(alerts_node->name, (const xmlChar *) "mailhost") == 0)
				{
					World.Alerts.mailhost = getcharfield(xmlNodeListGetString(doc,alerts_node->xmlChildrenNode, 1),NORMAL,NOBLANK);
				}
				else if (xmlStrcmp(alerts_node->name, (const xmlChar *) "mailhostport") == 0)
				{
					World.Alerts.mailhostport = getintfield(xmlNodeListGetString(doc,alerts_node->xmlChildrenNode, 1));
				}
				else if (xmlStrcmp(alerts_node->name, (const xmlChar *) "default_rcpt") == 0)
				{
					default_rcpt = getcharfield(xmlNodeListGetString(doc,alerts_node->xmlChildrenNode, 1),NORMAL,NOBLANK);
				}
				else if (xmlStrcmp(alerts_node->name, (const xmlChar *) "alert") == 0)
				{
					alert_node = alerts_node->xmlChildrenNode ;

					/* Alloc mem for struct */
					cur_alert = (alertPtr) malloc(sizeof(alert));
					checkmalloc(cur_alert);

					memset(cur_alert, 0, sizeof(alert));

					cur_alert->treshold = 0 ;
					cur_alert->dsname = NULL ;
					cur_alert->re_host = NULL ;
					cur_alert->re_program = NULL ;
					cur_alert->re_action = NULL ;
					cur_alert->mailrcpt = NULL ;
					cur_alert->mailsubject = NULL ;
					cur_alert->nb_hit_treshold = 0 ;
					cur_alert->last_alert = 0 ;

					while (alert_node != NULL)
					{
				 		if (xmlStrcmp(alert_node->name, (const xmlChar *) "host") == 0)
						{
							charptr = getcharfield(xmlNodeListGetString(doc,alert_node->xmlChildrenNode, 1),NORMAL,NOBLANK);
							cur_alert->re_host = regcomp(charptr) ;
							free(charptr);
						}
			 	
						else if (xmlStrcmp(alert_node->name, (const xmlChar *) "program") == 0)
						{
							charptr = getcharfield(xmlNodeListGetString(doc,alert_node->xmlChildrenNode, 1),NORMAL,NOBLANK);
							cur_alert->re_program = regcomp(charptr) ;
							free(charptr);
				
						}
				
						else if (xmlStrcmp(alert_node->name, (const xmlChar *) "action") == 0)
						{
							charptr = getcharfield(xmlNodeListGetString(doc,alert_node->xmlChildrenNode, 1),NORMAL,NOBLANK);
							cur_alert->re_action = regcomp(charptr) ;
							free(charptr);
						}
			 	
						else if (xmlStrcmp(alert_node->name, (const xmlChar *) "treshold") == 0)
						{ 
							cur_alert->dsname = getcharfield(xmlGetProp(alert_node, (const xmlChar *)"ds"),NORMAL,NOBLANK) ;
							cur_alert->treshold = getintfield(xmlNodeListGetString(doc,alert_node->xmlChildrenNode, 1));
						}
				
						else if (xmlStrcmp(alert_node->name, (const xmlChar *) "mailrcpt") == 0)
						{ 
							cur_alert->mailrcpt = getcharfield(xmlNodeListGetString(doc,alert_node->xmlChildrenNode, 1),NORMAL,NOBLANK);
						}
				
						else if (xmlStrcmp(alert_node->name, (const xmlChar *) "mailsubject") == 0)
						{ 
							cur_alert->mailsubject = getcharfield(xmlNodeListGetString(doc,alert_node->xmlChildrenNode, 1),NORMAL,FULL);
						}
					
						alert_node = alert_node->next ;

					} /*  fin du while */
					
				} /* fin du if */


				/* Stuff pour checker la struct alert et l ajouter a la liste chainee du world */
				if (cur_alert != NULL)
				{ 

					/* Check for subject */
					if (cur_alert->mailsubject == NULL) 
					{ 
							free_alert(cur_alert);
							alerts_node = alerts_node->next;
							continue ;
					}
					
					/* Check for rcpt */
					if (cur_alert->mailrcpt == NULL && default_rcpt == NULL)
					{
							free_alert(cur_alert);
							alerts_node = alerts_node->next;
							continue ;
					} 
					else if (cur_alert->mailrcpt == NULL && default_rcpt != NULL)
						cur_alert->mailrcpt = xstrdup(default_rcpt) ;


					/* L'alerte est du type Treshold */
					if ( (cur_alert->treshold > 0) && (cur_alert->dsname != NULL) )
					{ 
						if(World.Alerts.nbAlertTreshold == 0 ) 
						{
							World.Alerts.AlertTreshold = cur_alert ;
							alertptr = cur_alert ;
						}

						alertptr->next = cur_alert ;
						alertptr = alertptr->next ;	
						World.Alerts.nbAlertTreshold++ ;

						/* Juste pour etre sur ;=) */
						if (cur_alert->re_host) free(cur_alert->re_host);
						if (cur_alert->re_program) free(cur_alert->re_program);
						if (cur_alert->re_action) free(cur_alert->re_action);
					
					}
					/* L'alerte est du type Rule */
					else if ( (cur_alert->re_action != NULL) )
					{ 
						
						if(World.Alerts.nbAlertRules == 0 ) 
						{
							World.Alerts.AlertRules = cur_alert ;
							alertptr = cur_alert ;
						}

						alertptr->next = cur_alert ;
						alertptr = alertptr->next ;	
						World.Alerts.nbAlertRules++ ;
					
						/* Juste pour etre sur ;=) */
						if (cur_alert->dsname) free(cur_alert->dsname);
						cur_alert->treshold = 0 ;
					}
					/* La struct est imcomplete on free */
					else { 
							free_alert(cur_alert);
					}

				}
				
				/* On passe a l'alerte suivante */
				alerts_node = alerts_node->next;
			
			} /* fin du while */

			if (default_rcpt) free(default_rcpt);

			if (World.Alerts.mailhost == NULL)
				World.Alerts.mailhost = xstrdup(DEFAULT_MAILHOST);
			
			if (World.Alerts.mailhostport == 0)
				World.Alerts.mailhostport = DEFAULT_MAILHOSTPORT;
			
		}

		else if ((!xmlStrcasecmp(cur->name, (const xmlChar *) World.logformat))) 
		{   /* Parse Syslog Rules */
    	
			syslog_node = cur->xmlChildrenNode ;
	
			while (syslog_node != NULL) 
			{


			 	if (xmlStrcmp(syslog_node->name, (const xmlChar *) "rule") == 0) 
				{
					cur_rule = parseRule(doc,syslog_node);
	    			if (cur_rule != NULL)
					{

	    				if (World.nbRules == 0) 
						{ 
							World.rules = cur_rule ;
							ruleptr = cur_rule ;
						}

						World.nbRules++;
						ruleptr->next = cur_rule ;
						ruleptr = ruleptr->next ;
	    			}
				}

				syslog_node = syslog_node->next;
			}
		}
		
		else if ((!xmlStrcmp(cur->name, (const xmlChar *) "step"))) 
	    		World.step = getintfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
		
		else if ((!xmlStrcmp(cur->name, (const xmlChar *) "start"))) 
	    		World.start = getintfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
	
		else if ((!xmlStrcmp(cur->name, (const xmlChar *) "img_width"))) 
	    		World.width = getintfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));


		cur = cur->next;
    }
		

 
    /* Clean up everything else before quitting. */
    xmlCleanupParser();
	xmlFreeDoc(doc);
    
}


/* Parse the <rrd></rrd> content */
rrdDBPtr parse_rrdDB(xmlDocPtr doc, xmlNodePtr cur) { 
	
	rrdDBPtr DB = NULL ;
	dsPtr cur_ds = NULL , ds_ptr = NULL ; 
   
	/*  allocate the struct */
	DB = (rrdDBPtr) malloc(sizeof(rrdDB));
	checkmalloc(DB);

	memset(DB, 0, sizeof(rrdDB));
    
	/* Set some default values */
	DB->nbDs = 0 ;
	
	/* Get Down into the XMLTree ! */
   	cur = cur->xmlChildrenNode;
    	
	/* Search for our Items  */
   	while (cur != NULL) {

        
		if ((!xmlStrcmp(cur->name, (const xmlChar *) "rrdfile"))) {
	    		DB->rrdfile = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),NORMAL,NOBLANK);
		}
		else if ((!xmlStrcmp(cur->name, (const xmlChar *) "ds")))    {
	
				/* Malloc new DS chained list element */
				cur_ds = (dsPtr) malloc(sizeof(ds));
				checkmalloc(cur_ds);
				memset(cur_ds, 0, sizeof(ds));

				cur_ds->dsname = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),LOWERCASE,NOBLANK);
				cur_ds->dscount = 0 ;
			
	    		if (cur_ds != NULL) { 
				
					/* The rrdDB chained list */
					if (DB->nbDs == 0) {
						DB->ds = cur_ds ;
						ds_ptr = cur_ds ;
					}
				
					ds_ptr->next = cur_ds ;
					ds_ptr = ds_ptr->next ;
					DB->nbDs++;
	    		}
		}
		cur = cur->next ;
	}

	if (DB->nbDs == 0 || DB->rrdfile == NULL) { 
		freerrdDB(DB) ;
		return(NULL) ;
	} else {
		return(DB);
	}
}

/* Parse the <rule></rule> tag content */
rulesPtr parseRule(xmlDocPtr doc, xmlNodePtr cur) {
    
	rulesPtr Rule = NULL;
	char *charptr ;

   	/*
   	 * allocate the struct
   	 */

   	Rule = (rulesPtr) malloc(sizeof(rules));
	checkmalloc(Rule);
    
	memset(Rule, 0, sizeof(rules));

	/* Set default to break after a match */
    Rule->brk = 0 ;
	Rule->re_host = NULL ;
	Rule->re_program = NULL ;
	Rule->re_action = NULL ;

	/* Get Down into the XMLTree ! */
   	cur = cur->xmlChildrenNode;
	

   	while (cur != NULL) 
	{
       	if ((!xmlStrcmp(cur->name, (const xmlChar *) "program"))) 
		{
	    	charptr = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),NORMAL,NOBLANK);
			Rule->re_program = regcomp(charptr) ;
			free(charptr);
			if (Rule->re_program == NULL) 
			{ 
	    		freesyslogrule(Rule);
	    		return(NULL);
			}
		}
		else if ((!xmlStrcmp(cur->name, (const xmlChar *) "host"))) 
		{
	    	charptr = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),NORMAL,NOBLANK);
			Rule->re_host = regcomp(charptr) ;
			free(charptr);
			if (Rule->re_host == NULL) 
			{ 
	    		freesyslogrule(Rule);
	    		return(NULL);
			}
		}
		else if ((!xmlStrcmp(cur->name, (const xmlChar *) "action"))) 
		{
	    	charptr = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),NORMAL,NOBLANK);
			Rule->re_action = regcomp(charptr) ;
			free(charptr);
			if (Rule->re_action == NULL) 
			{ 
	    		freesyslogrule(Rule);
	    		return(NULL);
			}
		}
		else if ((!xmlStrcmp(cur->name, (const xmlChar *) "dsname"))) 
		{
	    	Rule->dsname = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),LOWERCASE,NOBLANK);
		}
		else if ((!xmlStrcmp(cur->name, (const xmlChar *) "break"))) 
		{
    		charptr = getcharfield(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1),LOWERCASE,NOBLANK);
    		if ( strcmp("false",charptr) == 0 ) 
			{
	    		Rule->brk = 1 ;
			}
    		if (charptr) { free(charptr); }
		}
			
			cur = cur->next;
   	}
    
	if (Rule->dsname == NULL || ( (Rule->re_program == NULL) && (Rule->re_host == NULL) && (Rule->re_action == NULL) ) ) { 
	    freesyslogrule(Rule);
	    return(NULL);
	} 
    
	return(Rule);
}
	    


/*
 *
 *	PRINT
 *
 *
 */



void Print_World() {
    
    printf("/* ===== * ===== */\n");
    printf("%d RRDdb registered\n", World.nbrrdDB);
    printf("%d Rules registered\n", World.nbRules);
    printf("%d AlertsRules registered\n", World.Alerts.nbAlertRules);
    printf("%d AlertsTreshold registered\n", World.Alerts.nbAlertTreshold);
    printf("/* ===== * ===== */\n\n");

}

void printds () { 

	struct Gds *DS ;
	u_short i ;
	
	DS = World.GDS ;
	for (i = 0 ; i < World.nbGDS  ; i++ ) 
	{ 
		printf("DS: %s , %d \n",DS->dsname_ptr,*DS->dscount_ptr);
		DS = DS->next ;
	}
}




/*
 *
 *	FREE
 *
 */

void freerrdDB (rrdDBPtr DB) { 
	unsigned short int i ;
	dsPtr ds_ptr ;
    
	ds_ptr = DB->ds ;

    for (i = 0; i < DB->nbDs; i++) {
			free(ds_ptr->dsname);
		    ds_ptr = ds_ptr->next ;
   	 }

}

void freesyslogrule(rulesPtr rule) { 
	free(rule->dsname) ;
	if (rule->re_action) pcre_free(rule->re_action);
	if (rule->re_host) pcre_free(rule->re_host);
	if (rule->re_program) pcre_free(rule->re_program);
}

void Free_World () {
	unsigned short int i ;
   	rulesPtr rule, ruleptr = NULL ;
	rrdDBPtr DB, DBptr=NULL ;
	alertPtr alert = NULL, alertptr = NULL ;   

	/* Free DS */
	DB = World.rrdDB ;
    for (i = 0; i < World.nbrrdDB; i++) { 
		freerrdDB(DB);
		DBptr = DB ;
		DB = DB->next ;
		free(DBptr);
	}
        
	/* Free Rules */
	rule = World.rules;
	for (i = 0; i < World.nbRules; i++) { 
		freesyslogrule(rule);
		ruleptr = rule ;
		rule = rule->next ;
		free(ruleptr) ;
	}

	/* Free Alerts */
	alert = World.Alerts.AlertRules ;
	for (i=0;i<World.Alerts.nbAlertRules;i++)
	{
		free_alert_rule(alert);
		alertptr = alert ; 
		alert = alert->next ;
		free(alertptr);
	}
	alert = World.Alerts.AlertTreshold ;
	for (i = 0; i < World.Alerts.nbAlertTreshold; i++)
	{
		free_alert_treshold(alert);
		alertptr = alert ; 
		alert = alert->next ;
		free(alertptr);
	}
	
	free(World.Alerts.mailhost);

	/* Free other Stuff */
	free(World.configfile);
	free(World.logfile);

	/* Free the regexp parser */
	pcre_free(World.Parser_Rule);

	free(World.logformat);

}


void free_alert (alertPtr alert)
{
	alert->treshold=0;
	if (alert->mailrcpt) free(alert->mailrcpt);
	if (alert->mailsubject) free(alert->mailsubject);
	if (alert->dsname) free(alert->dsname);
	if (alert->re_host) pcre_free(alert->re_host);
	if (alert->re_program) pcre_free(alert->re_program);
	if (alert->re_action) pcre_free(alert->re_action);
	free(alert);
	alert = NULL ;
}
void free_alert_treshold (alertPtr alert)
{
	alert->treshold=0;
	if (alert->mailrcpt) free(alert->mailrcpt);
	if (alert->mailsubject) free(alert->mailsubject);
	if (alert->dsname) free(alert->dsname);
}
void free_alert_rule (alertPtr alert)
{
	if (alert->mailrcpt) free(alert->mailrcpt);
	if (alert->mailsubject) free(alert->mailsubject);
	if (alert->re_host) pcre_free(alert->re_host);
	if (alert->re_program) pcre_free(alert->re_program);
	if (alert->re_action) pcre_free(alert->re_action);
}


/*
 *
 * MISC 
 *
 */

void Build_Global_DS_List () 
{

  	u_short i, j ;
	GdsPtr cur_ds = NULL , ds_ptr = NULL ;
	dsPtr ds = NULL ; 
	rrdDBPtr DB = NULL;
	
	DB = World.rrdDB ;

   for (i=0;i<World.nbrrdDB;i++) 
   { 

    ds = DB->ds ;
   	for (j = 0; j < DB->nbDs; j++)
	{
		/* Malloc new DS chained list element */
		cur_ds = (GdsPtr) malloc(sizeof(Gds));
		checkmalloc(cur_ds);
		memset(cur_ds, 0, sizeof(Gds));

		cur_ds->dsname_ptr = ds->dsname ; 
		cur_ds->dscount_ptr = &ds->dscount ;
			
		if (World.nbGDS == 0) 
		{
			World.GDS = cur_ds ;
			ds_ptr = cur_ds ;
		}
				
		ds_ptr->next = cur_ds ;
		ds_ptr = ds_ptr->next ;
		World.nbGDS++;
		ds = ds->next ;
	}
   DB = DB->next ;	
  }
}

